package math.numerics;

/**
 * LegendreInterpolator uses a polynomial interpolation formula to evaluate values between data points.
 *
 * @author W. Christian
 * @version 1.0
 */
public class LegendreInterpolator implements Function {
  /**
   * Polynomial coefficients.
   */
  protected double[] hornerCoef; // generalized Horner expansion coefficients
  double[] xd;
  double[] yd;


  /**
   * Constructs a Legendre interpolating polynomial from the given data.
   *
   * @param xdata double[]
   * @param ydata double[]
   */
  public LegendreInterpolator(double[] xdata, double[] ydata) {
    hornerCoef= new double[xdata.length];
    xd=xdata;
    yd=ydata;
    computeCoefficients(xdata, ydata);
  }

  private void computeCoefficients(double[] xd, double[] yd) {
    int n = xd.length;
    //coefficients = new double[size];
    for (int i = 0; i < n; i++)
      hornerCoef[i] = yd[i];
    n -= 1;
    for (int i = 0; i < n; i++) {
      for (int k = n; k > i; k--) {
        int k1 = k - 1;
        int kn = k - (i + 1);
        hornerCoef[k] = (hornerCoef[k] - hornerCoef[k1])/ (xd[k]- xd[kn]);
      }
    }
  }

  /**
   * Computes the interpolated y value for a given x value.
   * @param aNumber x value.
   * @return interpolated y value.
   */
  public double evaluate(double x) {
    int n = hornerCoef.length;
    double answer = hornerCoef[--n];
    while (--n >= 0)
      answer = answer * (x - xd[n]) + hornerCoef[n];
    return answer;
  }

  /**
   * Gets the polynomial coefficients c in
   * c[0] + c[1] * x + c[2] * x^2 + ....
   * This routine should be used with care because the Vandermonde matrix that is being
   * solved is ill conditioned.  See Press et al.
   *
   * @return double[]
   */
  public double[] getCoefficients() {
    int n = xd.length;
    double[] temp = new double[n];
    double[] coef = new double[n];
    temp[n-1] = -xd[0];
    for (int i = 1; i < n; i++) {
      for (int j = n - i-1; j < n - 1; j++)
        temp[j] -= xd[i] * temp[j + 1];
      temp[n-1] -= xd[i];
    }
    for (int j = 0; j < n; j++) {
      double a = n;
      for (int k = n-1; k >= 1; k--)
        a = k * temp[k] + xd[j] * a;
      double b = yd[j] / a;
      double c = 1.0;
      for (int k = n-1; k >= 0; k--) {
        coef[k] += c * b;
        c = temp[k] + xd[j] * c;
      }
    }
    return coef;
  }
}
